from dcim.svg.racks import get_device_name
from django import template
from utilities.html import foreground_color

register = template.Library()


@register.filter()
def rack_unit(value):
    if value % 1 == 0:
        return True
    else:
        return False


@register.filter()
def rack_unit_to_int(value):
    return int(value)


@register.filter()
def calculate_u_position(unit, rack):
    u_height = rack.u_height * 2
    height = int(unit.get("height", 1) * 2)
    unit_id = int(unit["id"] * 2)

    if rack.desc_units:
        return unit_id - 2
    else:
        if height > 1:
            return u_height - unit_id - height + 2
        else:
            return u_height - unit_id


@register.filter()
def mul(value, mul_value):
    return int(value) * mul_value


@register.filter()
def text_color(value):
    return foreground_color(value)


@register.filter()
def device_name(device):
    return get_device_name(device)



from netbox.plugins import PluginConfig


class NetboxReorderRackConfig(PluginConfig):
    name = "netbox_reorder_rack"
    verbose_name = "NetBox Reorder Rack"
    description = "NetBox plugin to reorder rack layouts."
    version = "1.1.1"
    base_url = "reorder"


config = NetboxReorderRackConfig



from netbox.api.routers import NetBoxRouter

from netbox_reorder_rack.api import views

router = NetBoxRouter()

router.register("save", views.SaveViewSet, basename="reorder")

urlpatterns = router.urls



from netbox.plugins import PluginTemplateExtension


class ReorderButton(PluginTemplateExtension):
    model = "dcim.rack"

    def buttons(self):
        return self.render("netbox_reorder_rack/inc/rack_button.html")


template_extensions = [ReorderButton]



from core.models import ObjectType
from dcim.models import Device
from dcim.models import DeviceRole
from dcim.models import DeviceType
from dcim.models import Manufacturer
from dcim.models import Rack
from dcim.models import Site
from users.models import ObjectPermission
from utilities.testing import TestCase


class ReorderRackAPITest(TestCase):
    @classmethod
    def setUpTestData(cls):
        site = Site.objects.create(name="Test Site")
        rack = Rack.objects.create(
            name="Test Rack 1", site=Site.objects.first(), u_height=42
        )
        manufacturer = Manufacturer.objects.create(name="Test Manufacturer")
        role = DeviceRole.objects.create(name="Test Role")
        device_type1 = DeviceType.objects.create(
            manufacturer=manufacturer,
            model="Test Device Type 1",
            slug="test-device-type-1",
            u_height=1,
            is_full_depth=True,
        )
        device_type2 = DeviceType.objects.create(
            manufacturer=manufacturer,
            model="Test Device Type 2",
            slug="test-device-type-2",
            u_height=5,
            is_full_depth=True,
        )
        device_type3 = DeviceType.objects.create(
            manufacturer=manufacturer,
            model="Test Device Type 3",
            slug="test-device-type-3",
            u_height=1,
            is_full_depth=False,
        )
        device1 = Device(
            name="Device 1",
            device_type=device_type1,
            site=site,
            rack=rack,
            position=1,
            face="front",
            role=role,
        )
        device2 = Device(
            name="Device 2",
            device_type=device_type2,
            site=site,
            rack=rack,
            position=2,
            face="front",
            role=role,
        )
        device3 = Device(
            name="Device 3",
            device_type=device_type3,
            site=site,
            rack=rack,
            position=40,
            face="rear",
            role=role,
        )
        device4 = Device(
            name="Device 4",
            device_type=device_type1,
            site=site,
            rack=rack,
            position=41,
            face="rear",
            role=role,
        )
        devices = [device1, device2, device3, device4]

        for device in devices:
            device.clean()
            device.save()

    def test_reorder_rack_view_without_permissions(self):
        rack = Rack.objects.get(name="Test Rack 1")
        device1 = Device.objects.get(name="Device 1")
        device2 = Device.objects.get(name="Device 2")
        device3 = Device.objects.get(name="Device 3")
        device4 = Device.objects.get(name="Device 4")

        data = {
            "front": [
                {
                    "id": device1.pk,
                    "x": 0,
                    "y": 42,
                    "is_full_depth": "True",
                    "face": "front",
                },
                {
                    "id": device2.pk,
                    "x": 0,
                    "y": 35,
                    "is_full_depth": "True",
                    "face": "front",
                },
                {
                    "id": device3.pk,
                    "x": 0,
                    "y": 1,
                    "is_full_depth": "False",
                    "face": "front",
                },
                {
                    "id": device4.pk,
                    "x": 0,
                    "y": 2,
                    "is_full_depth": "True",
                    "face": "front",
                },
            ],
            "rack_id": rack.pk,
            "rear": [],
            "other": [
                {
                    "id": device4.pk,
                    "x": 0,
                    "y": 2,
                    "is_full_depth": "True",
                    "face": None,
                }
            ],
        }

        resp = self.client.put(
            f"/api/plugins/reorder/save/{rack.pk}/",
            data,
            content_type="application/json",
        )
        self.assertHttpStatus(resp, 500)

    def test_reorder_rack_view_with_permissions(self):
        # Add model-level permission
        obj_perm = ObjectPermission(name="Test permission", actions=["view", "change"])
        obj_perm.save()
        obj_perm.users.add(self.user)
        obj_perm.object_types.add(ObjectType.objects.get_for_model(Device))

        rack = Rack.objects.get(name="Test Rack 1")
        device1 = Device.objects.get(name="Device 1")
        device2 = Device.objects.get(name="Device 2")
        device3 = Device.objects.get(name="Device 3")
        device4 = Device.objects.get(name="Device 4")

        data = {
            "front": [
                {
                    "id": device1.pk,
                    "x": 0,
                    "y": 42,
                    "is_full_depth": "True",
                    "face": "front",
                },
                {
                    "id": device2.pk,
                    "x": 0,
                    "y": 35,
                    "is_full_depth": "True",
                    "face": "front",
                },
                {
                    "id": device3.pk,
                    "x": 0,
                    "y": 1,
                    "is_full_depth": "False",
                    "face": "front",
                },
                {
                    "id": device4.pk,
                    "x": 0,
                    "y": 2,
                    "is_full_depth": "True",
                    "face": "front",
                },
            ],
            "rack_id": rack.pk,
            "rear": [],
            "other": [
                {
                    "id": device4.pk,
                    "x": 0,
                    "y": 2,
                    "is_full_depth": "True",
                    "face": None,
                }
            ],
        }

        resp = self.client.put(
            f"/api/plugins/reorder/save/{rack.pk}/",
            data,
            content_type="application/json",
        )
        self.assertHttpStatus(resp, 201)

        for num in [1, 2, 3, 4]:
            device = Device.objects.get(name=f"Device {num}")
            self.assertEqual(device.position, data["front"][num - 1]["y"])
            self.assertEqual(device.face, data["front"][num - 1]["face"])



from core.models import ObjectType
from dcim.models import Device
from dcim.models import Rack
from dcim.models import Site
from users.models import ObjectPermission
from utilities.testing import TestCase


class ReorderRackTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        site = Site.objects.create(name="Site 1", slug="site-1")
        Rack.objects.create(name="Rack 1", site=site, u_height=42)

    def test_reorder_rack_view_without_permissions(self):
        rack = Rack.objects.all().first()
        response = self.client.get(f"/dcim/racks/{rack.pk}/reorder/")
        self.assertHttpStatus(response, 403)

    def test_reorder_rack_view_with_permissions(self):
        # Add model-level permission
        obj_perm = ObjectPermission(name="Test permission", actions=["change", "view"])
        obj_perm.save()
        obj_perm.users.add(self.user)
        obj_perm.object_types.add(ObjectType.objects.get_for_model(Device))
        rack = Rack.objects.all().first()

        response = self.client.get(f"/dcim/racks/{rack.pk}/reorder/")
        self.assertHttpStatus(response, 200)



from dcim.models import Device
from dcim.models import Rack
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.shortcuts import get_object_or_404
from django.shortcuts import render
from django.views.generic import View
from utilities.views import register_model_view


@register_model_view(
    Rack,
    name="reorder",
    path="reorder",
)
class ReorderView(LoginRequiredMixin, PermissionRequiredMixin, View):
    permission_required = ["dcim.change_device", "dcim.view_device"]
    template_name = "netbox_reorder_rack/rack.html"

    def get(self, request, pk):
        rack = get_object_or_404(Rack, pk=pk)
        non_racked = Device.objects.filter(
            rack=rack, position__isnull=True, parent_bay__isnull=True
        )
        return render(
            request,
            self.template_name,
            {
                "object": rack,
                "front_units": rack.get_rack_units(expand_devices=False, face="front"),
                "rear_units": rack.get_rack_units(expand_devices=False, face="rear"),
                "non_racked": non_racked,
                "basepath": settings.BASE_PATH,
            },
        )



import decimal

from dcim.models import Device
from dcim.models import Rack
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.db import transaction
from django.shortcuts import get_object_or_404
from rest_framework import serializers
from rest_framework import status
from rest_framework import viewsets
from rest_framework.response import Response


class ReorderRackSerializer(serializers.Serializer):
    rack_id = serializers.IntegerField()
    front = serializers.ListField(child=serializers.JSONField())
    rear = serializers.ListField(child=serializers.JSONField())
    other = serializers.ListField(child=serializers.JSONField())


class SaveViewSet(PermissionRequiredMixin, viewsets.ViewSet):
    permission_required = ["dcim.change_device", "dcim.view_device"]
    serializer_class = ReorderRackSerializer
    queryset = Device.objects.none()
    schema = None

    def update(self, request, pk):
        rack = get_object_or_404(Rack, pk=pk)
        try:
            serializer = ReorderRackSerializer(request.data)
            with transaction.atomic():
                for device in rack.devices.all():
                    device.position = None
                    device.clean()
                    device.save()

                for new in request.data["front"]:
                    device = rack.devices.filter(pk=new["id"]).first()
                    device.position = decimal.Decimal(new["y"])
                    device.face = new["face"]
                    device.clean()
                    device.save()

                for new in request.data["rear"]:
                    device = rack.devices.filter(pk=new["id"]).first()
                    device.position = decimal.Decimal(new["y"])
                    device.face = new["face"]
                    device.clean()
                    device.save()

                return Response(
                    {"message": "POST request received", "data": serializer.data},
                    status=status.HTTP_201_CREATED,
                )
        except Exception as e:
            return Response(
                {"message": "Error saving data", "error": str(e)},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )



from setuptools import find_packages
from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="netbox_reorder_rack",
    version="1.1.1",
    author="Alex Gittings",
    author_email="agitting96@gmail.com",
    description="NetBox plugin to reorder rack layouts.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[],
    url="https://github.com/minitriga/netbox-reorder-rack/",
    project_urls={
        "Bug Tracker": "https://github.com/minitriga/netbox-reorder-rack/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Framework :: Django",
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)



from django.urls import include
from django.urls import path
from utilities.urls import get_model_urls

from netbox_reorder_rack.views import ReorderView  # noqa: F401

urlpatterns = (
    path(
        "reorder/<int:pk>/",
        include(get_model_urls("dcim", "rack")),
    ),
)
